@@ -17,15 +17,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $URL: svn://pegasos.dyndns.biz/praat/trunk/kNN/KNN_prune.c $
- * $Rev: 137 $
- * $Author: stix $
- * $Date: 2008-08-10 19:34:07 +0200 (Sun, 10 Aug 2008) $
- * $Id: KNN_prune.c 137 2008-08-10 17:34:07Z stix $
- */
-
 /*
  * os 20070529 Initial release
+ * os 20090123 Bugfix: Rejects classifiers containing 0 or 1 instances
  */
 
 #include "KNN_prune.h"
@@ -51,11 +45,18 @@ long KNN_prune_prune
 )
 
 {
+    Categories uniqueCategories = Categories_selectUniqueItems(my output, 1);
+    if(Categories_getSize(uniqueCategories) == my nInstances)
+        return(0);
+
     long removals = 0;
     long ncandidates = 0;
     long candidates[my nInstances];
     double progress = 1 / (double) my nInstances;
 
+    if(my nInstances <= 1)
+        return(0);
+
     for (long y = 1; y <= my nInstances; y++)
     {
         if (!Melder_progress1(1 - (double) y * progress, L"Pruning noisy instances")) return(removals);
@@ -64,14 +65,14 @@ long KNN_prune_prune
             if (n == 1 || NUMrandomUniform(0, 1) <= n)
             {
                 KNN_removeInstance(me, y);
-                removals++;
+                ++removals;
             }
         }
     }
 
     Melder_progress1(1.0, NULL);
 
-    for (long y = 1; y <= my nInstances; y++)
+    for (long y = 1; y <= my nInstances; ++y)
     {
         if (!Melder_progress1(1 - (double) y * progress, L"Identifying superfluous and critical instances")) return(removals);
         if (KNN_prune_superfluous(my input, my output, y, k, 0) && !KNN_prune_critical(my input, my output, y, k))
@@ -83,17 +84,20 @@ long KNN_prune_prune
     KNN_prune_sort(my input, my output, k, candidates, ncandidates);
     progress  = 1 / ncandidates;
 
-    for (long y = 0; y < ncandidates; y++)
+    for (long y = 0; y < ncandidates; ++y)
     {
         if (!Melder_progress1(1 - (double) y * progress, L"Pruning superfluous non-critical instances")) return(removals);
         if (KNN_prune_superfluous(my input, my output, candidates[y], k, 0) && !KNN_prune_critical(my input, my output, candidates[y], k))
         {
             if (r == 1 || NUMrandomUniform(0, 1) <= r)
             {
-                for (long yy = y + 1; yy < ncandidates; yy++)
-                    if (candidates[yy] > candidates[y]) candidates[yy]--;
                 KNN_removeInstance(me, candidates[y]);
-                removals++;
+
+                for(long i = y + 1; i < ncandidates; ++i)
+                    if(candidates[i] > candidates[y])
+                        --candidates[i];
+
+                ++removals;
             }
         }
     }
@@ -128,27 +132,30 @@ void KNN_prune_sort
     long n = nindices;
     long h[nindices];
 
-    for (long cc = 0; cc < nindices; cc++)
+    for (long cc = 0; cc < nindices; ++cc)
         h[cc] = KNN_friendsAmongkNeighbours(p, p, c, indices[cc], k);
 
-    while (--n)// insertion-sort, is heap-sort worth the effort?
+    while (--n) // insertion-sort, is heap-sort worth the effort?
     {
         for (long m = n; m < nindices - 1; m++)
         {
-            if (h[m - 1] > h[m]) break;
+            if (h[m - 1] > h[m]) 
+                break;
+
             if (h[m - 1] < h[m])
             {
-                LONGARRAYSWAP(indices, m - 1, m);
+                OlaSWAP(long, indices[m - 1], indices[m]);
             }
             else
             {
                 if (KNN_nearestEnemy(p, p, c, indices[m - 1]) < KNN_nearestEnemy(p, p, c, indices[m]))
                 {
-                    LONGARRAYSWAP(indices, m - 1, m);
+                    OlaSWAP(long, indices[m - 1], indices[m]);
                 }
-                else
+                else 
                 {
-                    if (NUMrandomUniform(0, 1) > 0.5) LONGARRAYSWAP(indices, m - 1, m);
+                    if (NUMrandomUniform(0, 1) > 0.5) 
+                        OlaSWAP(long, indices[m - 1], indices[m]);
                 }
             }
         }
@@ -179,8 +186,8 @@ long KNN_prune_kCoverage
 )
 
 {
-    if (y > p->ny) return(0);// safety belt
-    if (k > p->ny) k = p->ny;
+    Melder_assert(y <= p->ny);
+    Melder_assert(k > 0 && k <= p->ny);
 
     long cc = 0;
     FeatureWeights fws = FeatureWeights_create(p->nx);
@@ -192,7 +199,8 @@ long KNN_prune_kCoverage
         {
             if (y != yy && FRIENDS(c->item[y], c->item[yy]))
             {
-                long n = KNN_kNeighboursSkip(p, p, fws, yy, k, tempindices, 0);
+                // long n = KNN_kNeighboursSkip(p, p, fws, yy, k, tempindices, 0); .OS.081011
+                long n = KNN_kNeighboursSkip(p, p, fws, yy, k, tempindices, y);
                 while (n)
                 {
                     if (tempindices[--n] == y)
@@ -231,7 +239,7 @@ int KNN_prune_superfluous
 )
 
 {
-    if (y > p->ny) y = p->ny;// safety belt
+    if (y > p->ny) y = p->ny;   // safety belt
     if (k > p->ny) k = p->ny;
 
     FeatureWeights fws = FeatureWeights_create(p->nx);
@@ -243,7 +251,11 @@ int KNN_prune_superfluous
         double distances[k];
         double freqs[k];
 
-        KNN_kNeighboursSkip(p, p, fws, y, k, indices, skipper);
+        // KNN_kNeighboursSkip(p, p, fws, y, k, indices, skipper); .OS.081011 ->
+        if(!KNN_kNeighboursSkip(p, p, fws, y, k, indices, skipper))
+            return(0);
+        // .OS.081011 <-
+
         long ncategories = KNN_kIndicesToFrequenciesAndDistances(c, k, indices, distances, freqs, freqindices);
 
         forget(fws);
@@ -275,7 +287,7 @@ int KNN_prune_critical
 )
 
 {
-    if (y > p->ny) y = p->ny;// safety belt
+    if (y > p->ny) y = p->ny;   // safety belt
     if (k > p->ny) k = p->ny;
 
     FeatureWeights fws = FeatureWeights_create(p->nx);
@@ -283,7 +295,8 @@ int KNN_prune_critical
     if (fws)
     {
         long indices[k];
-        long ncollected = KNN_kNeighboursSkip(p, p, fws, y, k, indices, 0);
+        // long ncollected = KNN_kNeighboursSkip(p, p, fws, y, k, indices, 0); .OS.081011
+        long ncollected = KNN_kNeighboursSkip(p, p, fws, y, k, indices, y);
 
         for (long ic = 0; ic < ncollected; ic++)
             if (!KNN_prune_superfluous(p, c, indices[ic], k, 0) || !KNN_prune_superfluous(p, c, indices[ic], k, y))
@@ -317,14 +330,15 @@ int KNN_prune_noisy
 )
 
 {
-    if (y > p->ny) y = p->ny;// safety belt
+    if (y > p->ny) y = p->ny;   // safety belt
     if (k > p->ny) k = p->ny;
 
     FeatureWeights fws = FeatureWeights_create(p->nx);
     if (fws)
     {
-        long indices[p->ny];// the coverage is not bounded by k but by n
-        long reachability = KNN_kNeighboursSkip(p, p, fws, y, k, indices, 0);
+        long indices[p->ny];    // the coverage is not bounded by k but by n
+        // long reachability = KNN_kNeighboursSkip(p, p, fws, y, k, indices, 0); .OS.081011
+        long reachability = KNN_kNeighboursSkip(p, p, fws, y, k, indices, y); 
         long coverage = KNN_prune_kCoverage(p, c, y, k, indices);
 
         forget(fws);
